@@ -3,5 +3,5 @@ from .helpers import (
     get_annualized_variable,
     get_neutralized_variable,
 )
-from .variable import Variable
+from .variable import Variable, QuantityType
 from .typing import Formula
